from ak_file.sanitizer import (sanitize, _remove_blacklisted_characters, 
                                _remove_chars_below_code_point, obfuscate, unobfuscate)

def test_remove_blacklisted_characters():
    assert _remove_blacklisted_characters("a\\b/c:d*e?f\"g<h>i|j\0k") == "abcdefghijk"

def test_remove_chars_below_code_point():
    assert _remove_chars_below_code_point("abc\x1fdef", 32) == "abcdef"
    
def test_length():
    assert len(sanitize("gradsf"*250)) == 255
    assert len(sanitize("gra:d\\sf"*250)) == 255

def test_sanitize():
    # Test input containing all characters in blacklist
    assert sanitize("\\/:*?\"<>|\0") == "__"
    
    # Test input containing a reserved word
    assert sanitize("PRN") == "__PRN"
    
    # Test input containing only spaces and periods
    assert sanitize(" .  . .") == "__"
    
    # Test input containing only periods
    assert sanitize("...") == "__"
    
    # Test input with filename length greater than 255
    assert sanitize("a" * 300) == "a" * 255
    
    assert sanitize("") == '__'

def test_obfuscate():
    assert obfuscate('Filename to obfuscate') == 'WzCvErDvqKFqFswLJtrKv'
    assert obfuscate('This is a test file 1243, ??;', shift_val=23) == 'eEFPwFPwxwQBPQwCFIBwmnpo,w??;'

def test_unobfuscate():
    assert unobfuscate('WzCvErDvqKFqFswLJtrKv') == 'Filename to obfuscate'
    assert unobfuscate('eEFPwFPwxwQBPQwCFIBwmnpo,w??;', shift_val=23) == 'This is a test file 1243, ??;'


#!/usr/bin/env python
# coding: utf-8

from time import ctime
from pathlib import Path
import hashlib
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
import base64
from datetime import datetime
from dataclasses import dataclass

@dataclass
class FileProperties:
    atime: datetime
    mtime: datetime
    ctime: datetime
    size: int
    
class File:
    _DEFAULT_SALT = b'salt_^h.W#(e6-OHplcig:?6@+((8{_f2skE'
    
    def __init__(self, filepath: str|Path)-> None:
        self.filepath: Path = Path(str(filepath))

    def __str__(self) -> str:
        if props := self.properties:
            return (
                f'Name\t\t: {self.name}'
                f'\nDirectory\t: {self.parent}'
                '\nExists\t\t: True'
                f'\nAccess Time\t: {props.atime:%Y-%m-%d %H:%M:%S}'
                f'\nModified Time\t: {props.mtime:%Y-%m-%d %H:%M:%S}'
                f'\nChange Time\t: {props.ctime:%Y-%m-%d %H:%M:%S}'
                f'\nSize\t\t: {props.size/1024:.2f} KB'
            )
        else:
            return (
                f'Name        : {self.name}'
                f'\nDirectory   : {self.parent}'
                '\nExists          : False'
            )
            
    def __repr__(self) -> str:
        return f'File(filepath="{self.filepath}")'
    
    def __del__(self) -> bool:
        if self.filepath.exists():
            self.filepath.unlink()
            return True
        else:
            return False
    def __len__(self) -> int:
        """Returns size of file in Bytes"""
        if self.exists() and self.stat:
            return self.stat.st_size
        else:
            return 0
        
    def __gt__(self, other) -> bool:
        if isinstance(other, File):
            return len(self) > len(other)
        elif type(other) in (int, float):
            return len(self) > other
        else:
            raise TypeError(f"Unsupported comparison between instances of 'File' and '{other.__class__.__name__}'")  # noqa: E501

    def __lt__(self, other) -> bool:
        if isinstance(other, File):
            return len(self) < len(other)
        elif type(other) in (int, float):
            return len(self) < other
        else:
            raise TypeError(f"Unsupported comparison between instances of 'File' and '{other.__class__.__name__}'")  # noqa: E501

    def __ge__(self, other) -> bool:
        if isinstance(other, File):
            return len(self) >= len(other)
        elif type(other) in (int, float):
            return len(self) >= other
        else:
            raise TypeError(f"Unsupported comparison between instances of 'File' and '{other.__class__.__name__}'")  # noqa: E501

    def __le__(self, other) -> bool:
        if isinstance(other, File):
            return len(self) <= len(other)
        elif type(other) in (int, float):
            return len(self) <= other
        else:
            raise TypeError(f"Unsupported comparison between instances of 'File' and '{other.__class__.__name__}'")  # noqa: E501

    def __eq__(self, other) -> bool:
        """Checks Hash; If int supplied, checks size"""
        if isinstance(other, File):
            return self.hash == other.hash
        elif isinstance(other, str):
            return self.hash == other
        elif isinstance(other, int):
            return len(self) == other
        else:
            raise TypeError(f"Unsupported comparison between instances of 'File' and '{other.__class__.__name__}'")  # noqa: E501

    def __abs__(self) -> Path:
        return self.filepath.absolute()
    
    @property
    def hash(self) -> str:
        if self.filepath.exists():
            with open(self.filepath, 'rb') as f:
                _data: bytes = f.read()
            return hashlib.md5(_data).hexdigest()
        else:
            raise Exception(f"{self.filepath} does not exist.")
        
    @property
    def stat(self):
        if self.exists():
            return self.filepath.stat()
        else:
            return None
    
    @property
    def name(self) -> str:
        "Returns the filename from the current self.filepath"
        return self.filepath.name

    @property
    def parent(self) -> Path:
        "Returns the file directory from the current self.filepath"
        return self.filepath.absolute().parent
    
    @property
    def abspath(self) -> Path:
        "Returns absolute path of the current file"
        return self.filepath.absolute()
    
    def update_filename(self, new_filename:str, rename_file: bool = False) -> str:
        "Updates the filepath of the stored file"
        new_filepath = self.filepath.parent / new_filename
        if rename_file and self.exists():
            self.filepath.rename(new_filepath)
            self.filepath = new_filepath
        return self.name
    
    def is_file(self) -> bool:
        return self.filepath.is_file()

    def exists(self) -> bool:
        return self.filepath.is_file()
    
    @property
    def properties(self) -> FileProperties|None:
        """Returns a dict of properties of the file"""
        if self.stat:
            return FileProperties(
                atime=datetime.fromtimestamp(self.stat.st_atime),
                mtime=datetime.fromtimestamp(self.stat.st_mtime),
                ctime=datetime.fromtimestamp(self.stat.st_ctime),
                size=self.stat.st_size
            )
        return None
    
    def encrypt(self, key: str='') -> bytes|None:
        """Encrypts the content of the file"""
        content = self.read()
        if content:
            fernet = _generate_key(password = key, salt=self._DEFAULT_SALT)
            return fernet.encrypt(content)
        else: 
            return None
    
    def decrypt(self, key: str='') -> bytes|None:
        """Decrypts the content of the file"""
        content = self.read()
        if content:
            fernet = _generate_key(password = key, salt=self._DEFAULT_SALT)
            return fernet.decrypt(content)
        else: 
            return None
    
    def read(self) -> bytes | None:
        if self.exists() is None:
            return None
        else:
            with open(self.filepath, 'rb') as f:
                return f.read()
        
    def write(self, content: bytes) -> None:
        with open(self.filepath, 'wb') as f:
            f.write(content)

def _generate_key(password: str, salt: bytes) -> Fernet:
    """
    Generates a key from the given password and returns it.
    """
    password_bytes: bytes = bytes(password, 'utf-8')
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=390000,
        )
    key = base64.urlsafe_b64encode(kdf.derive(password_bytes))
    return Fernet(key)


import unicodedata
import re
from pathlib import Path
import string

MAX_FILENAME_LENGTH: int = 255
MAX_EXTENSION_LENGTH: int = 254
MIN_ASCII_VALUE: int = 32

def sanitize(filename:str, strict: bool = True) -> str:
    """Return a fairly safe version of the filename.
    """
    if not strict:
        filepath: Path = Path(filename)
        filedir: Path = filepath.parent
        filename = filepath.name
    else:
        filedir: Path = Path()
    
    filename = _remove_blacklisted_characters(filename)
    filename = _normalize_filename(filename)
    filename = _replace_whitespace(filename)
    filename = _remove_chars_below_code_point(filename, MIN_ASCII_VALUE)
    filename, ext = _handle_extension(filename)
    filename = _handle_reserved(filename)
    filename = _handle_max_length(filename, ext, 
                                    MAX_FILENAME_LENGTH, MAX_EXTENSION_LENGTH)

    return str(filedir / filename)

def _replace_whitespace(filename: str) -> str:
    return re.sub(r"\s+", '_', filename)

def _remove_blacklisted_characters(filename: str) -> str:
    """Remove blacklisted characters from the filename."""
    blacklist = ["\\", "/", ":", "*", "?", "\"", "<", ">", "|", "\0"]
    return "".join(c for c in filename if c not in blacklist)

def _remove_chars_below_code_point(filename: str, code_point: int) -> str:
    """Remove characters from the filename below a certain ASCII code point."""
    return "".join(c for c in filename if ord(c) > code_point)

def _normalize_filename(filename: str) -> str:
    """Normalize the filename and remove trailing/leading spaces and periods."""
    filename = unicodedata.normalize("NFKD", filename)
    filename = filename.rstrip(". ").strip()
    return "__" + filename if all([x == "." for x in filename]) else filename

def _handle_extension(filename: str) -> tuple[str, str]:
    """Handle the extension of the filename."""
    parts = re.split(r"/|\\", filename)[-1].split(".")
    ext = "." + parts.pop() if len(parts) > 1 else ""
    filename = filename[:-len(ext)] if ext else filename
    return filename, ext

def _handle_reserved(filename: str) -> str:
    """Handle reserved filenames."""
    
    # Reserved words on Windows
    reserved = [
        "CON", "PRN", "AUX", "NUL", "COM1", "COM2", "COM3", "COM4", "COM5",
        "COM6", "COM7", "COM8", "COM9", "LPT1", "LPT2", "LPT3", "LPT4", "LPT5",
        "LPT6", "LPT7", "LPT8", "LPT9",
    ]  
    return "__" + filename if filename in reserved else filename

def _handle_max_length(filename: str, ext: str, max_filename_length: int, 
                        max_extension_length: int) -> str:
    """Handle filenames that exceed the maximum length."""
    if len(filename) > max_filename_length:
        if len(ext) > max_extension_length:
            ext = ext[max_extension_length:]
        maxl: int = max_filename_length - len(ext)
        filename = filename[:maxl]
        filename = "__" if len(filename) == 0 else filename
    return filename + ext

def _stringshift(stringlist: str, shift: int) -> str:
    return stringlist[shift:] + stringlist[:shift]
    
def obfuscate(text: str, shift_val: int = 17) -> str:
    stringlist: str = f"{string.ascii_letters}0123456789. "
    
    replacement_dict: dict[str, str] = {}
    for original, replaced in zip(stringlist, _stringshift(stringlist, shift_val)):
        replacement_dict[original] = replaced
    
    new_text: str = ""
    for char in text:
        new_char = replacement_dict.get(char)
        if new_char is None:
            new_text += char
        else:
            new_text += new_char
    return new_text

def unobfuscate(text: str, shift_val: int = 17) -> str:
    stringlist: str = f"{string.ascii_letters}0123456789. "
    
    replacement_dict: dict[str, str] = {}
    for original, replaced in zip(stringlist, _stringshift(stringlist, shift_val)):
        replacement_dict[replaced] = original
    
    new_text: str = ""
    for char in text:
        new_char = replacement_dict.get(char)
        if new_char is None:
            new_text += char
        else:
            new_text += new_char
    return new_text


from pathlib import Path
from typing import Generator, Callable
import re
from datetime import datetime

class SearchFolder:
    def __init__(self, folder_path: str|Path, recurse: bool) -> None:
        self.folderpath: Path = Path(str(folder_path))
        self.recurse = recurse
        if not self.folderpath.exists() or not self.folderpath.is_dir():
            raise ValueError(f"{folder_path} does not exist or is not a directory.")

    def __str__(self) -> str:
        return f"Search in the directory: {self.folderpath}, Recurse = {self.recurse}"

    def __repr__(self) -> str:
        return f"Search('{self.folderpath}', {self.recurse})"
    
    def search(self, condition: Callable[[Path], bool]) -> Generator[Path, None, None]:
        """Recursively search for files that satisfy a condition."""
        for file in self.folderpath.glob(f"{'**/' if self.recurse else ''}*"):
            if file.is_file() and condition(file):
                yield file

    def size(self, min_size: int, max_size: int) -> Generator[Path, None, None]:
        """Search for files within a specific size range."""
        return self.search(lambda file: min_size <= file.stat().st_size <= max_size)

    def modification_date(self, start_date: datetime, 
                            end_date: datetime) -> Generator[Path, None, None]:
        """Search for files modified within a specific date range."""
        return self.search(lambda file: start_date <= datetime.fromtimestamp(file.stat().st_mtime) <= end_date)  # noqa: E501
    
    def regex(self, pattern: str, 
                case_sensitive:bool=False) -> Generator[Path, None, None]:
        """Search for files in a directory that match a regex pattern."""
        if (not case_sensitive) and (not pattern.startswith("(?i)")):
            pattern = "(?i)" + pattern
        regex = re.compile(pattern)
        return self.search(lambda file: regex.search(file.name) is not None)

    def extension(self, extension_str: str) -> Generator[Path, None, None]:
        """Search for files in a directory with a specific extension."""
        return self.folderpath.glob(f"{'**/' if self.recurse else ''}*.{extension_str}")
    